//////////////////// MODULE DEFINITION: Play Row Sound ////////////////////
//This module takes in the current position of a scan bar and outputs signals to control the DDS module
module PlayRowSound(
input CLK,
input reset,

//VGA controller position
input [9:0] vga_y,
input [9:0] vga_x,

//screen boundaries
input [7:0] xmin,
input [7:0] xmax,
input [8:0] row_y,
input [6:0] ymin,

input disp_bit, //current cell being read out of memory
input [2:0] mode, //controls tone mapping
input [23:0] period, //controls the speed
output play_sound, //triggers DDS module to play a sound
output [8:0] num_alive_out, //number of live cells in a row
output [3:0] tone_sel //octave for tones
); 
	reg [23:0] counter; //for controlling the speed
	reg [8:0] num_alive; //number of live cells in a row
	reg [2:0] state; //state machine variable
	reg [8:0] row_y_last; //used for detecting a new line
	reg play_soundR;
	reg [3:0] tone_selR;
	
	//random number generation
	reg [30:0] seed, randn; 
	wire rand_low_bit;
	assign rand_low_bit = randn[27] ^ randn[30];
	
	//map buffers
	assign tone_sel = tone_selR;
	assign play_sound = play_soundR;
	assign num_alive_out = num_alive;
	
	//states
	parameter waiting=3'd0, read_line0=3'd1, read_line1=3'd2, new_line=3'd3, toggle0=3'd4, toggle1=3'd5;
	
	always @ (posedge CLK) begin
		seed <= seed + 31'd1;
		if (reset) begin
			state <= waiting;
			num_alive <= 9'b0;
			tone_selR <= 4'd0;
			//random number generator
			randn <= seed;
			counter <= 24'd0;
		end
		else begin
			case (state)
				waiting: 
				begin
					row_y_last <= row_y;
					num_alive <= 9'b0;
					//if the bars are frozen, don't wait for a new line, simply hold for a period of time
					if ((mode == 3'd3) | (mode == 3'd4)) begin
						counter <= counter + 24'd1;
						if (counter >= period) state <= read_line0;
						else state <= waiting;
					end 
					else 
					begin
						//detect that the bar has moved to the next line
						if (row_y[8:2] != row_y_last[8:2]) state <= read_line0;
						else state <= waiting;
					end
				end
				
				read_line0:
				begin
					//wait until the VGA controller is on the beginning of the bar's line and begin counting
					if ((vga_y[8:2]==row_y[8:2]) & (vga_x[9:2]==xmin)) begin
						num_alive <= num_alive + {8'b0,disp_bit};
						state <= read_line1;
					end
					else state <= read_line0;
				end
				
				read_line1:
				begin
					//count until we reach the end of the row
					if ((vga_y[8:2]==row_y[8:2]) & (vga_x[9:2]<xmax)) begin
						num_alive <= num_alive + {8'b0,disp_bit};
						state <= read_line1; 
					end
					else state <= new_line;
				end
				
				new_line: 
				begin
					//if there were live cells then play a tone
					if (num_alive > 9'd0) state <= toggle0;
					else state <= waiting;
					num_alive <= num_alive;
				end
				
				toggle0:
				begin
															
					case (mode)
				
					3'd0: //tone mapping 0 - tone is proportial # of cells
					begin 
						play_soundR <= 1'b1;
						state <= toggle1;
						if (num_alive[8:2] < 7'd10) tone_selR <= 4'd0;
						else if (num_alive[8:2] < 7'd20) tone_selR <= 4'd1;
						else if (num_alive[8:2] < 7'd30) tone_selR <= 4'd2;
						else if (num_alive[8:2] < 7'd40) tone_selR <= 4'd3;
						else if (num_alive[8:2] < 7'd50) tone_selR <= 4'd4;
						else if (num_alive[8:2] < 7'd60) tone_selR <= 4'd5;
						else if (num_alive[8:2] < 7'd70) tone_selR <= 4'd6;
						else if (num_alive[8:2] < 7'd80) tone_selR <= 4'd7;
						else if (num_alive[8:2] < 7'd90) tone_selR <= 4'd8;
						else tone_selR <= 4'd9;
					end
					
					3'd1: //tone mapping 1 - tone is related to the low-order bits of the # of cells
					begin
						play_soundR <= 1'b1;
						state <= toggle1;
						if (num_alive[5:2] < 4'd1) tone_selR <= 4'd0;
						else if (num_alive[5:2] < 7'd2) tone_selR <= 4'd1;
						else if (num_alive[5:2] < 7'd4) tone_selR <= 4'd2;
						else if (num_alive[5:2] < 7'd6) tone_selR <= 4'd3;
						else if (num_alive[5:2] < 7'd8) tone_selR <= 4'd4;
						else if (num_alive[5:2] < 7'd10) tone_selR <= 4'd5;
						else if (num_alive[5:2] < 7'd11) tone_selR <= 4'd6;
						else if (num_alive[5:2] < 7'd12) tone_selR <= 4'd7;
						else if (num_alive[5:2] < 7'd14) tone_selR <= 4'd8;
						else tone_selR <= 4'd9;
					end
					
					3'd2: //tone mapping 2 - tone is related to low-order bits, but probability that it plays is determined by the # of live cells (more cells -> greater probabilit)
					begin
						state <= toggle1;
						randn <= {randn[29:0], rand_low_bit};
						
						if (randn[30:24] <= num_alive[8:2]) play_soundR <= 1'b1;
						else play_soundR <= 1'b0;
						
						if (num_alive[5:2] < 4'd1) tone_selR <= 4'd0;
						else if (num_alive[5:2] < 7'd2) tone_selR <= 4'd1;
						else if (num_alive[5:2] < 7'd4) tone_selR <= 4'd2;
						else if (num_alive[5:2] < 7'd6) tone_selR <= 4'd3;
						else if (num_alive[5:2] < 7'd8) tone_selR <= 4'd4;
						else if (num_alive[5:2] < 7'd10) tone_selR <= 4'd5;
						else if (num_alive[5:2] < 7'd11) tone_selR <= 4'd6;
						else if (num_alive[5:2] < 7'd12) tone_selR <= 4'd7;
						else if (num_alive[5:2] < 7'd14) tone_selR <= 4'd8;
						else tone_selR <= 4'd9;
					end
					
					//same as above 3'd2, but for fixed bars
					3'd3:
					begin
						state <= toggle1;
						randn <= {randn[29:0], rand_low_bit};
						
						if (randn[30:24] <= num_alive[8:2]) play_soundR <= 1'b1;
						else play_soundR <= 1'b0;
						
						if (num_alive[5:2] < 4'd1) tone_selR <= 4'd0;
						else if (num_alive[5:2] < 7'd2) tone_selR <= 4'd1;
						else if (num_alive[5:2] < 7'd4) tone_selR <= 4'd2;
						else if (num_alive[5:2] < 7'd6) tone_selR <= 4'd3;
						else if (num_alive[5:2] < 7'd8) tone_selR <= 4'd4;
						else if (num_alive[5:2] < 7'd10) tone_selR <= 4'd5;
						else if (num_alive[5:2] < 7'd11) tone_selR <= 4'd6;
						else if (num_alive[5:2] < 7'd12) tone_selR <= 4'd7;
						else if (num_alive[5:2] < 7'd14) tone_selR <= 4'd8;
						else tone_selR <= 4'd9;
					end

					//same as above 3'd1, but for fixed bars
					3'd4:
					begin
						play_soundR <= 1'b1;
						state <= toggle1;
						if (num_alive[5:2] < 4'd1) tone_selR <= 4'd0;
						else if (num_alive[5:2] < 7'd2) tone_selR <= 4'd1;
						else if (num_alive[5:2] < 7'd4) tone_selR <= 4'd2;
						else if (num_alive[5:2] < 7'd6) tone_selR <= 4'd3;
						else if (num_alive[5:2] < 7'd8) tone_selR <= 4'd4;
						else if (num_alive[5:2] < 7'd10) tone_selR <= 4'd5;
						else if (num_alive[5:2] < 7'd11) tone_selR <= 4'd6;
						else if (num_alive[5:2] < 7'd12) tone_selR <= 4'd7;
						else if (num_alive[5:2] < 7'd14) tone_selR <= 4'd8;
						else tone_selR <= 4'd9;
					end
					
					
					endcase
					
				end
				
				toggle1:
				begin
					//finish toggling the play_sound line and go back to waiting
					play_soundR <= 1'b0;
					state <= waiting;
					counter <= 24'd0;
				end
				
			endcase
			
		end

	end
endmodule