//////////////////// MODULE DEFINITION: Game of Life ////////////////////
module GameOfLife (
	input play, //play/pause game state
	input [3:0] obj, //new object to draw
	//cursor position
	input [9:0] cur_x, 
	input [9:0] cur_y,
	input reset, 
	input CLK,
	//edges of the screen
	input [7:0] xmax,
	input [7:0] xmin,
	input [6:0] ymax,
	input [6:0] ymin,
	//VGA controller position
	input [9:0] vga_x,
	input [9:0] vga_y,
	//memory interface
	input mem_in,
	input [23:0] update_period,
	output mem_out,
	output mem_we,
	output [18:0] mem_addr
);
	//Register Outputs
	reg mem_outR;
	reg [18:0] mem_addrR;
	reg mem_weR;
	assign mem_out = mem_outR;
	assign mem_addr = mem_addrR;
	assign mem_we = mem_weR;
	
	//Automaton State Machine Variables
	reg [3:0] state;	//state machine
	reg [7:0] curr_x, write_x, read_x; //current x position in creating a new image (160 cells)
	reg [6:0] curr_y, write_y, read_y; //current y position in creating a new image (120 lines)
	reg [156:0] line_prev,line_curr,line_next,line_new0,line_new1;
	reg [23:0] count; //for controlling the speed of the gol
	wire [3:0] num_neigh; //stores the number of neighbors of the current cell
	reg [3:0] objR; //object to draw
	reg [3:0] draw_state; //state of drawing SM
	reg [19:0] pixels_drawn; //counts total pixels drawn (for new objects)
	
	//store the position of the previous pixel draw
	reg [7:0] vga_x_last;
	reg [6:0] vga_y_last;
	
	wire cell_new, cell_curr; //store new and current values of a cell
	
	//set up random number generator
	wire rand_low_bit;
	reg [30:0] seed, randn;
	assign rand_low_bit = randn[27] ^ randn[30];

	//calculate the number of neighbors combinationally
	assign num_neigh = {3'b0,line_prev[156]} + {3'b0,line_prev[0]} + {3'b0,line_prev[1]} + {3'b0,line_curr[156]} + {3'b0,line_curr[1]} + {3'b0,line_next[156]} + {3'b0,line_next[0]} + {3'b0,line_next[1]};

	//holds current state of cell
	assign cell_curr = line_curr[0];

	//calculate new state of cell
	assign cell_new = (cell_curr) ? (((num_neigh < 4'd2) | (num_neigh > 4'd3)) ? 1'b0 : 1'b1) : ((num_neigh == 4'd3) ? 1'b1 : 1'b0); 

	//define states
	parameter store_update=4'd0, write_new_line=4'd1, read_new_line=4'd2, done=4'd3, init=4'd4;
	
	//draw objects
	parameter blank=4'd0, dot=4'd1, glider=4'd2, osc_sm=4'd3, osc_lg= 4'd4, explode=4'd5, clear_all=4'd6, rand=4'd7, gun=4'd8, ece5760=4'd9, chaos=4'd10, cross=4'd11;
	
	//draw states
	parameter done_draw=4'd0, drawing=4'd1;

	//State Machine Control
	always @ (posedge CLK) 
	begin
		vga_x_last <= vga_x[9:2];
		vga_y_last <= vga_y[8:2];
		seed <= seed + 31'd1;
		if (reset) //synch reset assumes KEY0 is held down 1/60 second
		begin		
			//initialize control registers
			curr_x <= xmin; write_x <= xmin; read_x <= xmin;
			curr_y <= ymin; write_y <= ymin; read_y <= ymin;
			line_prev <= 156'b0; line_curr <= 156'b0; line_next <= 156'b0;
			line_new0 <= 156'b0; line_new1 <=156'b0;
			
			//initialize drawing registers (for when GOL is paused)
			pixels_drawn <= 20'd0; draw_state <= 4'd0; objR <= 4'd0;

			count <= 24'h0;
			
			//initialize screen
			mem_addrR <= {vga_y[8:2],vga_x[9:2]} ;
			mem_weR <= 1'b1;
			
			//random number generator
			randn <= seed;
			
			//draw starting cells:
			//square and line
			if ((vga_x[9:2]==8'd80) | (vga_x[9:2]==8'd81) | (vga_x[9:2]==8'd82) | (vga_x[9:2]==8'd20)) begin
				if ((vga_y[8:2]==7'd60) | (vga_y[8:2]==7'd61) | (vga_y[8:2]==7'd62)) mem_outR <= 1'b1;
				else mem_outR <= 1'b0;	
			end
			
			//glider 1
			else if ((vga_x[9:2]==8'd25) & (vga_y[8:2]==7'd20)) mem_outR <= 1'b1;
			else if ((vga_x[9:2]==8'd26) & (vga_y[8:2]==7'd20)) mem_outR <= 1'b1;
			else if ((vga_x[9:2]==8'd27) & (vga_y[8:2]==7'd20)) mem_outR <= 1'b1;
			else if ((vga_x[9:2]==8'd27) & (vga_y[8:2]==7'd19)) mem_outR <= 1'b1;
			else if ((vga_x[9:2]==8'd26) & (vga_y[8:2]==7'd18)) mem_outR <= 1'b1;
			
			//glider 2
			else if ((vga_x[9:2]==8'd30) & (vga_y[8:2]==7'd10)) mem_outR <= 1'b1;
			else if ((vga_x[9:2]==8'd31) & (vga_y[8:2]==7'd10)) mem_outR <= 1'b1;
			else if ((vga_x[9:2]==8'd32) & (vga_y[8:2]==7'd10)) mem_outR <= 1'b1;
			else if ((vga_x[9:2]==8'd30) & (vga_y[8:2]==7'd9)) mem_outR <= 1'b1;
			else if ((vga_x[9:2]==8'd31) & (vga_y[8:2]==7'd8)) mem_outR <= 1'b1;
			
			//glider 3
			else if ((vga_x[9:2]==8'd90) & (vga_y[8:2]==7'd100)) mem_outR <= 1'b1;
			else if ((vga_x[9:2]==8'd91) & (vga_y[8:2]==7'd100)) mem_outR <= 1'b1;
			else if ((vga_x[9:2]==8'd92) & (vga_y[8:2]==7'd100)) mem_outR <= 1'b1;
			else if ((vga_x[9:2]==8'd90) & (vga_y[8:2]==7'd99)) mem_outR <= 1'b1;
			else if ((vga_x[9:2]==8'd91) & (vga_y[8:2]==7'd98)) mem_outR <= 1'b1;
			
			//exploder
			else if ((vga_x[9:2]==8'd40) & ((vga_y[8:2]==7'd100)|(vga_y[8:2]==7'd101)|(vga_y[8:2]==7'd102)|(vga_y[8:2]==7'd103)|(vga_y[8:2]==7'd104))) mem_outR <= 1'b1;
			else if ((vga_x[9:2]==8'd42) & ((vga_y[8:2]==7'd100)|(vga_y[8:2]==7'd104))) mem_outR <= 1'b1;
			else if ((vga_x[9:2]==8'd44) & ((vga_y[8:2]==7'd100)|(vga_y[8:2]==7'd101)|(vga_y[8:2]==7'd102)|(vga_y[8:2]==7'd103)|(vga_y[8:2]==7'd104))) mem_outR <= 1'b1;
			
			//otherwise black
			else mem_outR <= 1'b0;
			
			state <= init;
		end
		
		//begin state machine to modify display 
		else begin
			case(state)		
				
				init:
				begin
					//start read
					state <= read_new_line;
					mem_weR <= 1'b0;
					mem_addrR <= {read_y,read_x};
				end
				
				store_update:
				begin
					if (curr_x < xmax) begin
						curr_x <= curr_x + 8'd1;
						
						//circular shift registers
						line_new1 <= {cell_new,line_new1[156:1]}; //store new cell value
						line_prev <= {line_prev[0], line_prev[156:1]}; 
						line_curr <= {line_curr[0], line_curr[156:1]};
						line_next <= {line_next[0], line_next[156:1]};
						
						state <= store_update;
					end
					else if (curr_y < (ymax + 7'd1)) begin
						curr_y <= curr_y + 7'd1;
						curr_x <= xmin; 
						
						//swap registers
						line_prev <= line_curr;
						line_curr <= line_next;
						
						//write back the oldest line (line_new0)
						state <= write_new_line;
						write_y <= curr_y - 7'd1;
						write_x <= xmin;
						read_y <= curr_y + 7'd2;
						read_x <= xmin;
					end
					
					else begin
						state <= done;
					end
				end
				
				write_new_line: 
				begin
					if ((write_x < xmax)  & (write_y >= ymin)) begin
						write_x <= write_x + 8'b1;
						mem_weR <= 1'b1; //enable writing
						mem_addrR <= {write_y,write_x};	
						//hacked way to fix left-boundary, also use line_new0[2] or else image drifts
						mem_outR <= ((write_x==xmin) | (write_x==(xmin-8'd1)) | (write_y==ymin) | (write_y==(ymin-7'd1))) ? 1'b0 : (line_new0[2]);
						line_new0 <= {line_new0[0],line_new0[156:1]}; 
						
						state <= write_new_line;
					end
					else begin
						state <= read_new_line;
						mem_weR <= 1'b0;
						mem_addrR <= {read_y,read_x};
						line_new0 <= line_new1;
					end
				end
				
				read_new_line:
				begin
					if (read_x < xmax) begin
						read_x <= read_x + 8'b1;
						mem_weR <= 1'b0; //disable writing
						mem_addrR <= {read_y,read_x};
						line_next <= {mem_in, line_next[156:1]}; //store read cells into the next line registser
						
						state <= read_new_line;
					end
					else begin
						//after reading the new line, update the current line of cells
						state <= store_update;
					end
				end
				
				done: 
				begin
					if (play==1'b1) begin 
					//if game isn't paused, wait specified period of time before allowing the automaton to evolve
						if (count < update_period) begin
							state <= done;
							count <= count + 24'd1;
						end
						else begin
							count <= 24'h0;
							//initialize control registers
							curr_x <= xmin; write_x <= xmin; read_x <= xmin + 8'd1;
							curr_y <= ymin; write_y <= ymin; read_y <= ymin;
							line_prev <= 156'b0; line_curr <= 156'b0; line_next <= 156'b0;
							line_new0 <= 156'b0; line_new1 <=156'b0;
							
							//start read of 1st pixel
							state <= read_new_line;
							mem_weR <= 1'b0;
							mem_addrR <= {ymin,xmin};
						end 
					end
					
					else begin //game is paused, allow user to add new cells
					
						mem_addrR <= {vga_y[8:2],vga_x[9:2]} ; //the write address is the same as the VGA controller 
						
						case (draw_state) 
						
						done_draw: 
						begin 
							objR <= obj; //store new object to draw
							mem_weR <= 1'b0;
							pixels_drawn <= 20'd0;
							
							if (objR != obj) draw_state <= drawing;
							else draw_state <= done_draw;
						end
						
						drawing: 
						begin
							//check that cursor is in GOL region							
							if ((cur_x[9:2] > xmin) & (cur_x[9:2] < xmax) & (cur_y[8:2] >= ymin) & (cur_y[8:2] < ymax)) begin
								if (vga_x[9:2] != vga_x_last)
								begin
								
								case (objR)
								
								blank:
								begin
									draw_state <= done_draw;
								end
								
								dot: //draw a single cell at cursor position
								begin 
									if (pixels_drawn == 20'b1) draw_state <= done_draw;
									else draw_state <= drawing; 
									
									if ((vga_x[9:2]==cur_x[9:2]) & (vga_y[8:2]==cur_y[8:2])) begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn[0] <= 1'd1; end
									
									//otherwise don't write to memory
									else mem_weR <= 1'b0;
								end
								
								glider: //draw a glider at cursor location
								begin
									//update state variable
									if (pixels_drawn == 20'b11111) draw_state <= done_draw;
									else draw_state <= drawing; 
									
									//draw glider at the cursor's position
									if ((vga_x[9:2]==cur_x[9:2]) & (vga_y[8:2]==cur_y[8:2])) begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn[0] <= 1'd1; end
									else if ((vga_x[9:2]==(cur_x[9:2]+8'd1)) & (vga_y[8:2]==cur_y[8:2])) begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn[1] <= 1'd1; end
									else if ((vga_x[9:2]==(cur_x[9:2]+8'd2)) & (vga_y[8:2]==cur_y[8:2])) begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn[2] <= 1'd1; end
									else if ((vga_x[9:2]==(cur_x[9:2]+8'd2)) & (vga_y[8:2]==(cur_y[8:2]-7'd1))) begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn[3] <= 1'd1; end
									else if ((vga_x[9:2]==(cur_x[9:2]+8'd1)) & (vga_y[8:2]==(cur_y[8:2]-7'd2))) begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn[4] <= 1'd1; end
									
									//otherwise don't write to memory
									else mem_weR <= 1'b0;
								end 
								
								osc_sm: //draw a small oscillator at the cursor location
								begin
									if (pixels_drawn == 20'b111) draw_state <= done_draw;
									else draw_state <= drawing; 
									
									if ((vga_x[9:2]==cur_x[9:2]) & (vga_y[8:2]==cur_y[8:2])) begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn[0] <= 1'd1; end
									else if ((vga_x[9:2]==(cur_x[9:2]+8'd1)) & (vga_y[8:2]==cur_y[8:2])) begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn[1] <= 1'd1; end
									else if ((vga_x[9:2]==(cur_x[9:2]+8'd2)) & (vga_y[8:2]==cur_y[8:2])) begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn[2] <= 1'd1; end
									
									//otherwise don't write to memory
									else mem_weR <= 1'b0;
								end
								
								osc_lg: //draw a large oscillator at the cursor location
								begin
									if (pixels_drawn >= 20'd20) draw_state <= done_draw; //number of pixels drawn * 4
									else draw_state <= drawing;

									if ((vga_x[9:2]>=cur_x[9:2]) & (vga_x[9:2]<=cur_x[9:2]+4'd4) & (vga_y[8:2]==cur_y[8:2])) begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
																	
									//otherwise don't write to memory
									else mem_weR <= 1'b0;
								end
								
								explode: //draw an exploder at the cursor location
								begin 
	
									if (pixels_drawn >= 20'd48) draw_state <= done_draw; //number of pixels drawn * 4
									else draw_state <= drawing;

									if ((vga_x[9:2]==cur_x[9:2]) & (vga_y[8:2]>=cur_y[8:2]) & (vga_y[8:2]<=(cur_y[8:2]+7'd4))) begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									else if ((vga_x[9:2]==(cur_x[9:2]+8'd4)) & (vga_y[8:2]>=cur_y[8:2]) & (vga_y[8:2]<=(cur_y[8:2]+7'd4))) begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									else if ((vga_x[9:2]==(cur_x[9:2]+8'd2)) & ((vga_y[8:2]==cur_y[8:2]) | (vga_y[8:2]==(cur_y[8:2]+7'd4)))) begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									
									//otherwise don't write to memory
									else mem_weR <= 1'b0;
									
								end
								
								chaos: //draw a simple structure that results in chaos (15 cells)
								begin 
	
									if (pixels_drawn >= 20'd60) draw_state <= done_draw; //number of pixels drawn * 4
									else draw_state <= drawing;

									case ({vga_x[9:2],vga_y[8:2]})
										{(cur_x[9:2]+(8'd4)),(cur_y[8:2]+(7'd1))}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(cur_x[9:2]+(8'd2)),(cur_y[8:2]+(7'd2))}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(cur_x[9:2]+(8'd3)),(cur_y[8:2]+(7'd2))}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(cur_x[9:2]+(8'd4)),(cur_y[8:2]+(7'd2))}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(cur_x[9:2]+(8'd5)),(cur_y[8:2]+(7'd2))}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(cur_x[9:2]+(8'd6)),(cur_y[8:2]+(7'd2))}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(cur_x[9:2]+(8'd1)),(cur_y[8:2]+(7'd3))}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(cur_x[9:2]+(8'd2)),(cur_y[8:2]+(7'd3))}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(cur_x[9:2]+(8'd4)),(cur_y[8:2]+(7'd3))}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(cur_x[9:2]+(8'd6)),(cur_y[8:2]+(7'd3))}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(cur_x[9:2]+(8'd7)),(cur_y[8:2]+(7'd3))}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(cur_x[9:2]+(8'd2)),(cur_y[8:2]+(7'd4))}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(cur_x[9:2]+(8'd4)),(cur_y[8:2]+(7'd4))}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(cur_x[9:2]+(8'd6)),(cur_y[8:2]+(7'd4))}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(cur_x[9:2]+(8'd4)),(cur_y[8:2]+(7'd5))}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										default: mem_weR <= 1'b0;
									endcase
									
								end
								
								cross: //draw an "X" across the entire screen
								begin 
	
									if (pixels_drawn >= 20'd930) draw_state <= done_draw; //number of pixels drawn * 4
									else draw_state <= drawing;

									case ({vga_x[9:2],vga_y[8:2]})
										{(8'd21),(7'd1)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd132),(7'd1)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd22),(7'd2)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd131),(7'd2)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd23),(7'd3)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd130),(7'd3)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd24),(7'd4)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd129),(7'd4)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd25),(7'd5)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd128),(7'd5)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd26),(7'd6)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd127),(7'd6)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd27),(7'd7)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd7)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd28),(7'd8)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd8)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd29),(7'd9)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd124),(7'd9)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd10)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd123),(7'd10)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd11)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd122),(7'd11)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd32),(7'd12)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd121),(7'd12)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd33),(7'd13)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd120),(7'd13)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd34),(7'd14)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd119),(7'd14)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd35),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd118),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd36),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd117),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd37),(7'd17)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd116),(7'd17)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd38),(7'd18)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd115),(7'd18)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd39),(7'd19)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd114),(7'd19)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd40),(7'd20)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd113),(7'd20)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd41),(7'd21)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd112),(7'd21)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd42),(7'd22)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd111),(7'd22)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd43),(7'd23)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd23)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd24)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd24)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd25)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd108),(7'd25)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd46),(7'd26)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd107),(7'd26)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd47),(7'd27)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd106),(7'd27)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd28)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd105),(7'd28)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd29)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd104),(7'd29)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd50),(7'd30)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd103),(7'd30)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd51),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd102),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd52),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd53),(7'd33)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd33)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd54),(7'd34)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd99),(7'd34)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd55),(7'd35)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd35)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd56),(7'd36)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd36)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd57),(7'd37)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd96),(7'd37)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd58),(7'd38)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd95),(7'd38)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd59),(7'd39)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd94),(7'd39)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd60),(7'd40)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd93),(7'd40)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd61),(7'd41)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd92),(7'd41)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd62),(7'd42)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd91),(7'd42)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd63),(7'd43)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd90),(7'd43)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd64),(7'd44)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd89),(7'd44)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd65),(7'd45)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd88),(7'd45)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd66),(7'd46)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd87),(7'd46)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd67),(7'd47)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd86),(7'd47)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd68),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd69),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd70),(7'd50)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd83),(7'd50)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd51)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd82),(7'd51)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd52)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd81),(7'd52)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd73),(7'd53)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd80),(7'd53)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd54)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd79),(7'd54)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd55)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd78),(7'd55)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd76),(7'd56)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd77),(7'd56)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd76),(7'd57)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd77),(7'd57)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd58)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd78),(7'd58)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd59)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd79),(7'd59)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd73),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd80),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd81),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd62)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd82),(7'd62)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd70),(7'd63)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd83),(7'd63)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd69),(7'd64)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd64)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd68),(7'd65)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd65)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd67),(7'd66)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd86),(7'd66)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd66),(7'd67)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd87),(7'd67)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd65),(7'd68)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd88),(7'd68)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd64),(7'd69)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd89),(7'd69)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd63),(7'd70)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd90),(7'd70)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd62),(7'd71)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd91),(7'd71)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd61),(7'd72)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd92),(7'd72)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd60),(7'd73)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd93),(7'd73)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd59),(7'd74)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd94),(7'd74)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd58),(7'd75)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd95),(7'd75)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd57),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd96),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd56),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd55),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd54),(7'd79)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd99),(7'd79)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd53),(7'd80)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd80)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd52),(7'd81)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd81)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd51),(7'd82)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd102),(7'd82)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd50),(7'd83)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd103),(7'd83)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd84)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd104),(7'd84)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd85)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd105),(7'd85)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd47),(7'd86)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd106),(7'd86)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd46),(7'd87)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd107),(7'd87)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd88)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd108),(7'd88)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd89)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd89)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd43),(7'd90)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd90)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd42),(7'd91)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd111),(7'd91)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd41),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd112),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd40),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd113),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd39),(7'd94)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd114),(7'd94)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd38),(7'd95)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd115),(7'd95)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd37),(7'd96)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd116),(7'd96)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd36),(7'd97)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd117),(7'd97)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd35),(7'd98)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd118),(7'd98)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd34),(7'd99)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd119),(7'd99)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd33),(7'd100)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd120),(7'd100)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd32),(7'd101)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd121),(7'd101)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd102)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd122),(7'd102)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd103)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd123),(7'd103)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd29),(7'd104)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd124),(7'd104)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd28),(7'd105)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd105)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd27),(7'd106)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd106)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd26),(7'd107)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd127),(7'd107)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd25),(7'd108)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd128),(7'd108)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd24),(7'd109)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd129),(7'd109)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd23),(7'd110)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd130),(7'd110)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd22),(7'd111)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd131),(7'd111)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd21),(7'd112)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd132),(7'd112)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd20),(7'd113)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd133),(7'd113)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd19),(7'd114)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd134),(7'd114)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd18),(7'd115)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd135),(7'd115)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd17),(7'd116)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd136),(7'd116)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										default: mem_weR <= 1'b0;
									endcase

									
								end
								
								gun: //draw a glider gun
								begin
									if (pixels_drawn >= 20'd140) draw_state <= done_draw; //number of pixels drawn * 4
									else draw_state <= drawing;
									
									case ({vga_x[9:2],vga_y[8:2]}) 
									
									{(cur_x[9:2]+8'd0),(cur_y[8:2]+7'd0)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd0),(cur_y[8:2]+7'd1)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd1),(cur_y[8:2]+7'd0)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd1),(cur_y[8:2]+7'd1)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd8),(cur_y[8:2]+7'd1)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd8),(cur_y[8:2]+7'd2)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd9),(cur_y[8:2]+7'd0)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd9),(cur_y[8:2]+7'd2)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd10),(cur_y[8:2]+7'd0)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd10),(cur_y[8:2]+7'd1)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd16),(cur_y[8:2]+7'd2)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd16),(cur_y[8:2]+7'd3)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd16),(cur_y[8:2]+7'd4)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd17),(cur_y[8:2]+7'd2)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd18),(cur_y[8:2]+7'd3)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd22),(cur_y[8:2]-7'd1)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd22),(cur_y[8:2]+7'd0)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd23),(cur_y[8:2]-7'd2)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd23),(cur_y[8:2]+7'd0)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd24),(cur_y[8:2]-7'd2)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd24),(cur_y[8:2]-7'd1)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd24),(cur_y[8:2]+7'd10)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd24),(cur_y[8:2]+7'd11)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd25),(cur_y[8:2]+7'd10)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd25),(cur_y[8:2]+7'd12)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd26),(cur_y[8:2]+7'd10)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd34),(cur_y[8:2]-7'd1)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd34),(cur_y[8:2]-7'd2)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd35),(cur_y[8:2]-7'd1)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd35),(cur_y[8:2]-7'd2)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd35),(cur_y[8:2]+7'd5)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd35),(cur_y[8:2]+7'd6)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd35),(cur_y[8:2]+7'd7)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd36),(cur_y[8:2]+7'd5)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									{(cur_x[9:2]+8'd37),(cur_y[8:2]+7'd6)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
									default: mem_weR <= 1'b0;
									endcase
								
								end
								
								clear_all: //clear the whole screen
								begin
									if (pixels_drawn >= 20'hFFFFF) draw_state <= done_draw;
									else draw_state <= drawing; 
									
									if (vga_x[9:2]>xmin & vga_x[9:2]<xmax & vga_y[8:2]>=ymin & vga_y[8:2]<ymax) begin
										mem_outR <= 1'b0; 
										mem_weR <= 1'b1; 
										pixels_drawn <= pixels_drawn + 20'd1;
									end
									
									//otherwise don't write to memory
									else mem_weR <= 1'b0;
									
								end
								
								rand: //draw random cells using the rand register
								begin
									if (pixels_drawn >= 20'h3FFFF) draw_state <= done_draw;
									else draw_state <= drawing; 
									
									if ((vga_x[9:2]>xmin) & (vga_x[9:2]<xmax) & (vga_y[8:2]>ymin) & (vga_y[8:2]<ymax) & (vga_x[1:0]==2'b0) & (vga_y[1:0]==2'b0)) begin
										mem_outR <= randn[30]; 
										randn <= {randn[29:0], rand_low_bit};
										mem_weR <= 1'b1; 
										pixels_drawn <= pixels_drawn + 20'd1;
									end
									
									//otherwise don't write to memory
									else mem_weR <= 1'b0;
									
								end
								
								ece5760: //draw a pattern in the shape of "ECE 5760"
								begin
									if (pixels_drawn >= 20'd4368) draw_state <= done_draw;
									else draw_state <= drawing;
									case ({vga_x[9:2],vga_y[8:2]})
										{(8'd44),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd46),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd47),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd50),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd51),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd52),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd53),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd54),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd55),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd56),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd57),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd58),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd59),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd60),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd61),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd62),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd73),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd76),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd77),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd78),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd79),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd80),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd81),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd82),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd83),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd86),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd87),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd88),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd89),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd99),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd102),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd103),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd104),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd105),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd106),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd107),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd108),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd111),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd112),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd113),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd114),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd115),(7'd15)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd46),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd47),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd50),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd51),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd52),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd53),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd54),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd55),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd56),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd57),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd58),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd59),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd60),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd61),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd62),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd73),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd76),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd77),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd78),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd79),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd80),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd81),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd82),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd83),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd86),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd87),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd88),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd89),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd99),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd102),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd103),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd104),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd105),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd106),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd107),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd108),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd111),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd112),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd113),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd114),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd115),(7'd16)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd17)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd17)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd17)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd17)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd17)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd17)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd18)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd18)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd18)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd18)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd18)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd18)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd19)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd19)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd19)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd19)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd19)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd19)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd20)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd20)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd20)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd20)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd20)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd20)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd21)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd21)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd21)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd21)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd21)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd21)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd22)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd22)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd22)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd22)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd22)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd22)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd23)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd23)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd23)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd23)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd23)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd23)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd24)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd24)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd24)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd24)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd24)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd24)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd25)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd25)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd25)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd25)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd25)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd25)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd26)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd26)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd26)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd26)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd26)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd26)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd27)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd27)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd27)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd27)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd27)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd27)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd28)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd28)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd28)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd28)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd28)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd28)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd29)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd29)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd29)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd29)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd29)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd29)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd30)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd30)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd30)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd30)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd30)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd30)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd46),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd47),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd50),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd51),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd52),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd53),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd54),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd55),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd56),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd57),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd58),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd99),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd102),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd103),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd104),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd105),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd106),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd107),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd108),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd111),(7'd31)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd46),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd47),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd50),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd51),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd52),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd53),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd54),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd55),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd56),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd57),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd58),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd99),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd102),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd103),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd104),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd105),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd106),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd107),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd108),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd111),(7'd32)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd33)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd33)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd33)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd33)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd33)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd33)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd34)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd34)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd34)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd34)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd34)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd34)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd35)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd35)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd35)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd35)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd35)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd35)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd36)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd36)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd36)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd36)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd36)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd36)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd37)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd37)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd37)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd37)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd37)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd37)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd38)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd38)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd38)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd38)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd38)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd38)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd39)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd39)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd39)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd39)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd39)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd39)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd40)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd40)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd40)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd40)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd40)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd40)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd41)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd41)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd41)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd41)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd41)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd41)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd42)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd42)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd42)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd42)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd42)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd42)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd43)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd43)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd43)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd43)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd43)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd43)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd44)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd44)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd44)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd44)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd44)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd44)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd45)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd45)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd45)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd45)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd45)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd45)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd46)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd46)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd46)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd46)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd46)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd46)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd47)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd47)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd47)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd47)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd47)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd47)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd46),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd47),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd50),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd51),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd52),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd53),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd54),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd55),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd56),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd57),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd58),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd59),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd60),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd61),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd62),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd73),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd76),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd77),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd78),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd79),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd80),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd81),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd82),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd83),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd86),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd87),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd88),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd89),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd99),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd102),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd103),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd104),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd105),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd106),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd107),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd108),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd111),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd112),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd113),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd114),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd115),(7'd48)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd46),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd47),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd50),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd51),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd52),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd53),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd54),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd55),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd56),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd57),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd58),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd59),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd60),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd61),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd62),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd73),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd76),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd77),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd78),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd79),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd80),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd81),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd82),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd83),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd86),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd87),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd88),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd89),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd99),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd102),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd103),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd104),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd105),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd106),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd107),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd108),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd111),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd112),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd113),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd114),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd115),(7'd49)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd32),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd33),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd34),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd35),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd36),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd37),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd38),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd39),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd40),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd41),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd42),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd43),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd46),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd47),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd50),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd59),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd60),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd61),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd62),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd63),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd64),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd65),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd66),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd67),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd68),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd69),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd70),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd73),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd111),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd112),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd113),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd114),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd115),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd116),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd117),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd118),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd119),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd120),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd121),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd122),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd123),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd124),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd60)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd32),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd33),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd34),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd35),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd36),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd37),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd38),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd39),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd40),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd41),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd42),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd43),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd46),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd47),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd50),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd59),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd60),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd61),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd62),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd63),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd64),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd65),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd66),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd67),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd68),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd69),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd70),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd71),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd72),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd73),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd111),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd112),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd113),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd114),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd115),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd116),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd117),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd118),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd119),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd120),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd121),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd122),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd123),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd124),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd61)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd62)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd62)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd62)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd62)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd62)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd62)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd62)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd62)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd62)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd62)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd63)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd63)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd63)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd63)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd63)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd63)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd63)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd63)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd63)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd63)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd64)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd64)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd64)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd64)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd64)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd64)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd64)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd64)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd64)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd64)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd65)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd65)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd65)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd65)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd65)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd65)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd65)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd65)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd65)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd65)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd66)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd66)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd66)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd66)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd66)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd66)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd66)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd66)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd66)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd66)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd67)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd67)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd67)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd67)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd67)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd67)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd67)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd67)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd67)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd67)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd68)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd68)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd68)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd68)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd68)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd68)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd68)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd68)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd68)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd68)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd69)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd69)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd69)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd69)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd69)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd69)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd69)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd69)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd69)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd69)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd70)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd70)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd70)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd70)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd70)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd70)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd70)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd70)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd70)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd70)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd71)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd71)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd71)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd71)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd71)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd71)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd71)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd71)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd71)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd71)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd72)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd72)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd72)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd72)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd72)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd72)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd72)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd72)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd72)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd72)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd73)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd73)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd73)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd73)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd73)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd73)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd73)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd73)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd73)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd73)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd74)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd74)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd74)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd74)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd74)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd74)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd74)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd74)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd74)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd74)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd75)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd75)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd75)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd75)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd75)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd75)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd75)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd75)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd75)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd75)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd32),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd33),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd34),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd35),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd36),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd37),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd38),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd39),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd40),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd41),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd42),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd43),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd46),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd47),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd76)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd32),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd33),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd34),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd35),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd36),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd37),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd38),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd39),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd40),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd41),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd42),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd43),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd46),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd47),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd86),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd87),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd88),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd89),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd90),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd91),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd92),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd93),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd94),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd95),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd96),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd99),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd77)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd86),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd87),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd88),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd89),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd90),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd91),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd92),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd93),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd94),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd95),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd96),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd99),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd78)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd79)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd79)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd79)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd79)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd79)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd79)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd79)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd79)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd79)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd79)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd79)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd79)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd80)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd80)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd80)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd80)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd80)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd80)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd80)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd80)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd80)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd80)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd80)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd80)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd81)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd81)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd81)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd81)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd81)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd81)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd81)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd81)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd81)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd81)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd81)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd81)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd82)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd82)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd82)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd82)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd82)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd82)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd82)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd82)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd82)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd82)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd82)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd82)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd83)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd83)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd83)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd83)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd83)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd83)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd83)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd83)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd83)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd83)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd83)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd83)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd84)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd84)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd84)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd84)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd84)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd84)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd84)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd84)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd84)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd84)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd84)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd84)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd85)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd85)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd85)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd85)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd85)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd85)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd85)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd85)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd85)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd85)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd85)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd85)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd86)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd86)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd86)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd86)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd86)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd86)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd86)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd86)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd86)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd86)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd86)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd86)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd87)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd87)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd87)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd87)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd87)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd87)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd87)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd87)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd87)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd87)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd87)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd87)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd88)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd88)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd88)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd88)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd88)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd88)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd88)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd88)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd88)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd88)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd88)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd88)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd89)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd89)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd89)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd89)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd89)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd89)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd89)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd89)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd89)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd89)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd89)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd89)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd90)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd90)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd90)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd90)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd90)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd90)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd90)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd90)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd90)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd90)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd90)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd90)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd91)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd91)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd91)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd91)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd91)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd91)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd91)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd91)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd91)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd91)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd91)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd91)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd32),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd33),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd34),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd35),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd36),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd37),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd38),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd39),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd40),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd41),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd42),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd43),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd46),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd47),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd86),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd87),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd88),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd89),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd90),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd91),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd92),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd93),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd94),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd95),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd96),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd99),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd111),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd112),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd113),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd114),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd115),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd116),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd117),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd118),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd119),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd120),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd121),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd122),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd123),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd124),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd92)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd30),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd31),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd32),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd33),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd34),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd35),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd36),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd37),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd38),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd39),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd40),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd41),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd42),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd43),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd44),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd45),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd46),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd47),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd48),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd49),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd74),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd75),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd84),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd85),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd86),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd87),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd88),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd89),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd90),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd91),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd92),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd93),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd94),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd95),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd96),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd97),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd98),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd99),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd100),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd101),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd109),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd110),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd111),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd112),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd113),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd114),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd115),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd116),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd117),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd118),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd119),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd120),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd121),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd122),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd123),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd124),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd125),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										{(8'd126),(7'd93)}: begin mem_outR <= 1'b1; mem_weR <= 1'b1; pixels_drawn <= pixels_drawn+20'd1; end
										default: mem_weR <= 1'b0;
									endcase


							end
								
								endcase
								end
								
								else begin mem_weR <= 1'b0; draw_state <= drawing; end
							end
							
							//otherwise don't write to memory
							else begin mem_weR <= 1'b0; draw_state <= done_draw; end
						end 
						
						endcase
					end
				end
			endcase
		end 
		
	end // always @ (posedge CLK)

endmodule 